unit Unit1;
{$I TeeDefs.inc}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeeGDIPlus, StdCtrls, TeeProcs, TeEngine, Chart, ExtCtrls,
  TeeComma, Series, TeeOpenGL, TeCanvas, TeeGLCanvas, TeeTools, ComCtrls,
  {$IFDEF D12}
  TeeD2Canvas,
  {$ENDIF}
  TeeGLEditor, TeeJavaScript, jpeg;

type
  // Small class just to do an infinite loop:
  TUpdateThread=class(TThread)
  private
    IForm : TForm;
  public
    procedure Execute; override;
  end;

  TForm1 = class(TForm)
    TeeCommander1: TTeeCommander;
    Panel1: TPanel;
    Chart1: TChart;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    ListBox1: TListBox;
    TeeOpenGL1: TTeeOpenGL;
    Timer1: TTimer;
    ChartTool1: TRepaintMonitor;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    CBMode: TCheckBox;
    Label1: TLabel;
    TrackBar1: TTrackBar;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Image1: TImage;
    ComboFlat1: TComboFlat;
    Label5: TLabel;
    CBSync: TCheckBox;
    CBParallel: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure Chart1Resize(Sender: TObject);
    procedure Chart1Scroll(Sender: TObject);
    procedure Chart1Zoom(Sender: TObject);
    procedure Chart1UndoZoom(Sender: TObject);
    procedure Chart1AfterDraw(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure ComboFlat1Change(Sender: TObject);
    procedure CBSyncClick(Sender: TObject);
    procedure TeeOpenGL1Init(Sender: TObject);
    procedure CBParallelClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    IThread : TUpdateThread;

    Started,
    FirstRun : Boolean; // Flag for OpenGL VSync only

    procedure AddSeries;
    procedure AppIdle(Sender: TObject; var Done:Boolean);
    procedure DoStuff;
    procedure RefreshTotal;
    procedure ReuseXValues(DoReuse:Boolean);
    procedure UpdateData;
    procedure UpdateSeries(Index:Integer);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// Number of data points per Series:
var
  Num_Points:Integer=100000;

// Create several series and add lots of random data:
procedure TForm1.AddSeries;
var t : Integer;
    tmp : TChartValue;
    tmpMin,tmpMax,
    tt : Integer;
begin
  Chart1.FreeAllSeries;

  for t:=0 to TrackBar1.Position-1 do
  with Chart1.AddSeries(TFastLineSeries) as TFastLineSeries do
  begin
    // Much faster to draw "All" points using Polyline, than
    // drawing each point as a separate line segment:
    DrawStyle:=flAll;

    // Applies to "old" GDI canvas only, increases paint speed:
    FastPen:=True;

    // Initialize series X and Y values:

    XValues.Count:=Num_Points;

    // Optimization:
    // As we are creating all Series with the same XValues, we can simply
    // reuse the first series XValues array into all the other series:

    if t=0 then
       SetLength(XValues.Value,Num_Points)
    else
       XValues.Value:=Chart1[0].XValues.Value;

    YValues.Count:=Num_Points;
    SetLength(YValues.Value,Num_Points);

    // Add Num_Points random data to series arrays:

    tmp:=t*1000+Random(400);

    // Use a 1000 range:
    tmpMin:=t*1000;
    tmpMax:=(t+1)*1000;

    for tt:=0 to Num_Points-1 do
    begin
      // X value is simply the point index.
      // Here we optimize filling the array only for the first Series,
      // as all series share the same X values:
      if t=0 then
         XValues.Value[tt]:=tt;

      tmp:=tmp+Random(60)-29.5;

      // Check random data is inside the range we want (1000)
      if tmp>tmpMax then
         tmp:=tmpMax
      else
      if tmp<tmpMin then
         tmp:=tmpMin;

      // Set Y value:
      YValues.Value[tt]:=tmp;
    end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  // (FastCalc Applies to 32bit only).
  // Use 32bit assembler to increase speed:
  Chart1.Axes.FastCalc:=True;

  Chart1.Legend.Hide;

  // Enable or disable multi-CPU usage (speeds up FastLine series with "dsAll" DrawStyle)
  CBParallelClick(Self);

  {$IFDEF D12}
  ListBox1.Items.Add('Direct2D');
  {$ENDIF}

  ListBox1.ItemIndex:=0;

  AddSeries;

  // Hide grid lines:
  Chart1.Axes.Bottom.Grid.Hide;
  Chart1.Axes.Left.Grid.Hide;

  Chart1.Axes.Bottom.Texts.Selected.Hover.Font.Color:=clWhite;
  Chart1.Axes.Left.Texts.Selected.Hover.Font.Color:=clWhite;

  // Disable OpenGL lighting to increase speed:
  TeeOpenGL1.Light0.Visible:=False;
  TeeOpenGL1.ShadeQuality:=False;
  TeeOpenGL1.AmbientLight:=0; // <-- 0 means Full Light (Ambient = from 0 to 100)

  // Do not paint bevels:
  Chart1.BevelOuter:=bvNone;

  ListBox1Click(Self);

  // Cosmetic change colors and text font colors:
  Chart1.Color:=clBlack;
  Chart1.Title.Font.Color:=RGBA(200,150,50,0);
  Chart1.Axes.Left.Texts.Font.Color:=clWhite;
  Chart1.Axes.Bottom.Texts.Font.Color:=clWhite;
end;

procedure TForm1.AppIdle(Sender: TObject; var Done:Boolean);
begin
  if not Started then
  begin
    CheckBox4Click(Self);
    Started:=True;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  Application.OnIdle:=AppIdle;
end;

type
  TFastAccess=class(TFastLineSeries);

// Speed optimization:
// Enable or disable reusing the X coordinates, to avoid recalculating them.
procedure TForm1.ReuseXValues(DoReuse:Boolean);
var t : Integer;
begin
  for t:=0 to Chart1.SeriesCount-1 do
      TFastAccess(Chart1[t]).ReuseXValues:=DoReuse;
end;

procedure TForm1.Chart1Resize(Sender: TObject);
begin
  // When zooming or scrolling, or resizing the chart, force X coordinates
  // recalculation again:
  ReuseXValues(False);
end;

procedure TForm1.Chart1AfterDraw(Sender: TObject);
begin
  // Speed optimization:
  // After chart is displayed, tell all Series to conservate and reuse their
  // X coordinates, to avoid recalculating them again:
  ReuseXValues(True);
end;

procedure TForm1.Chart1Scroll(Sender: TObject);
begin
  Chart1Resize(Self);
end;

procedure TForm1.Chart1UndoZoom(Sender: TObject);
begin
  Chart1Resize(Self);
end;

procedure TForm1.Chart1Zoom(Sender: TObject);
begin
  Chart1Resize(Self);
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  Chart1.Axes.FastCalc:=CheckBox1.Checked;
end;

procedure TForm1.ListBox1Click(Sender: TObject);
begin
  FirstRun:=True;
  
  // Change Chart Canvas class (OpenGL, GDI or GDI+)

  TeeOpenGL1.Active:=ListBox1.ItemIndex=0;

  if ListBox1.ItemIndex=1 then
     Chart1.Canvas:=TTeeCanvas3D.Create
  else
  if ListBox1.ItemIndex=2 then
     Chart1.Canvas:=TGDIPlusCanvas.Create
  else
  if ListBox1.ItemIndex=3 then
  begin
    Chart1.Canvas:=TGDIPlusCanvas.Create;
    TGDIPlusCanvas(Chart1.Canvas).AntiAlias:=False;
  end
  {$IFDEF D12}
  else
  if ListBox1.ItemIndex=4 then
    Chart1.Canvas:=TTeeD2Canvas.Create
  {$ENDIF};

  // VSync only applies to OpenGL canvas:
  if not TeeOpenGL1.Active then
     CBSync.Enabled:=False;
end;

procedure TForm1.CheckBox2Click(Sender: TObject);
var t : Integer;
begin
  // FastPen applies to "old" GDI canvas only:
  for t:=0 to Chart1.SeriesCount-1 do
      TFastLineSeries(Chart1[t]).FastPen:=CheckBox2.Checked;
end;

procedure TForm1.CheckBox3Click(Sender: TObject);
var t : Integer;
begin
  // When DrawAllPoints = False, it means repeated X position points will
  // not be painted.
  for t:=0 to Chart1.SeriesCount-1 do
      TFastLineSeries(Chart1[t]).DrawAllPoints:=CheckBox3.Checked;
end;

procedure TForm1.CheckBox4Click(Sender: TObject);
begin
  // Switch between using a TTimer or a TThread to do the infinite loop:

  Timer1.Enabled:=not CheckBox4.Checked;

  if CheckBox4.Checked then
  begin
    IThread:=TUpdateThread.Create(True);
    IThread.FreeOnTerminate:=True;
    IThread.Priority:=tpNormal;
    IThread.IForm:=Self;

    {$IFDEF D9}
    IThread.Start;
    {$ELSE}
    IThread.Resume;
    {$ENDIF}
  end
  else
  if Assigned(IThread) then
     IThread.Terminate;
end;

procedure TForm1.UpdateSeries(Index:Integer);
const
  BufferSize=100;

var tmpMin,
    tmpMax,
    t : Integer;
    tmp0 : Array[0..BufferSize-1] of TChartValue;
    val : TChartValue;
    tmp : TChartValues;
begin
  with Chart1[Index] do
  begin
    tmp:=YValues.Value;

    // In "Buffer mode", move data points to simulate data scrolling

    if CBMode.Checked then
    begin
      // Backup the first points:
      for t := 0 to BufferSize-1 do
          tmp0[t]:=tmp[t];

      // Scroll data from right to left:
      for t := 0 to Num_Points-1-BufferSize do
          tmp[t]:=tmp[t+BufferSize];

      // Restore the first points to the last:
      for t := 0 to BufferSize-1 do
          tmp[Num_Points-1-BufferSize+t]:=tmp0[t];

    end
    else
    begin
      tmpMin:=Index*1000;
      tmpMax:=(Index+1)*1000;

      // Modify all values, increase them with a random quantity:
      for t:=0 to Num_Points-1 do
      begin
        val:=tmp[t]+Random(32)-15.5;

        if (val<tmpMax) and (val>tmpMin) then
           tmp[t]:=val;
      end;
    end;
  end;
end;

// Modify all series, all points "Y" values:

procedure TForm1.UpdateData;
var t : Integer;
begin
  if CBParallel.Checked then
     TTeeCPU.ParallelFor(0,Chart1.SeriesCount-1,UpdateSeries)
  else
  for t:=0 to Chart1.SeriesCount-1 do
      UpdateSeries(t);
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  UpdateData;
  Chart1.Invalidate;
end;

procedure TForm1.TrackBar1Change(Sender: TObject);
begin
  // Change the number of series:
  Label2.Caption:=IntToStr(TrackBar1.Position);

  RefreshTotal;
  AddSeries;
end;

procedure TForm1.DoStuff;
begin
  UpdateData;
  Chart1.Invalidate;
  Application.ProcessMessages;
end;

{ TUpdateThread }

// Simple infinite loop
procedure TUpdateThread.Execute;
begin
  inherited;

  while not Terminated do
  begin
    Synchronize(TForm1(IForm).DoStuff);
  end;
end;

procedure TForm1.Image1Click(Sender: TObject);
begin
  TeeGotoURL(Handle,'http://www.steema.com');
end;

procedure TForm1.RefreshTotal;
begin
  Label4.Caption:=FormatFloat('#,###',TrackBar1.Position*Num_Points);
end;

procedure TForm1.ComboFlat1Change(Sender: TObject);
begin
  Num_Points:=StrToInt(ComboFlat1.Text);

  RefreshTotal;
  AddSeries;
end;

// Enable or disable OpenGL Monitor Vertical Sync:
procedure TForm1.CBSyncClick(Sender: TObject);
begin
  if CBSync.Checked then
     TeeOpenGL1.Canvas.ScreenSync:=ssYes
  else
     TeeOpenGL1.Canvas.ScreenSync:=ssNo;
end;

procedure TForm1.TeeOpenGL1Init(Sender: TObject);
begin
  if FirstRun then
  begin
    CBSync.Enabled:=TeeOpenGL1.Active and (TeeOpenGL1.Canvas.ScreenSync<>ssSystem);

    if CBSync.Enabled then
    begin
     TeeOpenGL1.Canvas.ScreenSync:=ssNo;

     CBSync.Checked:=(TeeOpenGL1.Canvas.ScreenSync=ssYes);
    end;

    FirstRun:=False;
  end;
end;

type
  TChartAccess=class(TCustomAxisPanel);

// Experimental:
// Use all available CPU processors to calculate all Series data in parallel.
procedure TForm1.CBParallelClick(Sender: TObject);
begin
  if CBParallel.Checked then
     TChartAccess(Chart1).ParallelThreads:=TTeeCPU.NumberOfProcessors
  else
     TChartAccess(Chart1).ParallelThreads:=1;
end;

end.
